package com.zfire.jiasm.syncdata.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.zfire.jiasm.syncdata.service.MasterDownloadService;
import com.zfire.jiasm.syncdata.utils.DataUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.CallableStatementCallback;
import org.springframework.jdbc.core.CallableStatementCreator;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
public class MasterDownloadServiceImpl implements MasterDownloadService {

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@Override
	public List<Map<String, Object>> getTaskData() {
		List<Map<String, Object>> result = jdbcTemplate
				.queryForList("SELECT * FROM itf_gree_to_fa_download_task where need_download=1 order by id ");
		return result;
	}

	private String genInsertSql(int parameterLength, String destTabName, String destTabColList) {

		StringBuffer mark = new StringBuffer();
		for (int i = 0; i < parameterLength; i++) {
			if (mark.length() > 0) {
				mark.append(",");
			}
			mark.append("?");
		}

		return "insert into " + destTabName + " (" + destTabColList + ",syn_task_no) values (" + mark + ") ";
	}
	
	@Transactional(rollbackFor = Exception.class)
	public void finish(int downloadTaskType,String afterProcName,String synTaskNo,String belongCompany) throws Exception{
		
		Map<String, Object> result = jdbcTemplate.execute(new CallableStatementCreator() {

			public CallableStatement createCallableStatement(Connection con) throws SQLException {

				CallableStatement cs = con.prepareCall("{call " + afterProcName + "(?,?,?,?,?)}");
				cs.setInt(1, downloadTaskType);
				cs.setString(2, synTaskNo);
				cs.setString(3, belongCompany);
				cs.registerOutParameter(4, Types.BOOLEAN);
				cs.registerOutParameter(5, Types.VARCHAR);

				return cs;
			}
		}, new CallableStatementCallback<Map<String, Object>>() {

			public Map<String, Object> doInCallableStatement(CallableStatement cs)
					throws SQLException, DataAccessException {

				cs.execute();

				boolean status = cs.getBoolean(4);
				String errorMessage = cs.getString(5);

				Map<String, Object> result = new HashMap<String, Object>();
				result.put("status", status);
				result.put("errorMessage", errorMessage);

				return result;

			}
		});

		if (!(Boolean) result.get("status")) {
			throw new SQLException("执行过程[" + afterProcName + "]发生错误[" + result.get("errorMessage") + "]");
		}
		
	}

	@Transactional(rollbackFor = Exception.class)
	public void insertData(String srcTabColList, String destTabName, String destTabColList,
			String synTaskNo, JSONArray jsonArray) throws Exception {
		if (jsonArray != null) {
			String[] arySrcTabCol = srcTabColList.split("\\Q,\\E");
			Object[] arySrcTabColValue = new Object[arySrcTabCol.length + 1];
			String sql = genInsertSql(arySrcTabColValue.length, destTabName, destTabColList);
			for (int i = 0; i < jsonArray.size(); i++) {
				JSONObject aData = jsonArray.getJSONObject(i);
				for (int j = 0; j < arySrcTabCol.length; j++) {
					arySrcTabColValue[j] = findValue(DataUtil.fromDbName2ServiceName(arySrcTabCol[j].trim()), aData);
				}
				arySrcTabColValue[arySrcTabColValue.length - 1] = synTaskNo;
				jdbcTemplate.update(sql, arySrcTabColValue);
			}
		}
	}

	private String findValue(String srcTabCol, JSONObject aData) {

		Set<String> set = aData.keySet();
		for (String key : set) {
			if (key.equalsIgnoreCase(srcTabCol)) {
				return aData.getString(key);
			}
		}

		return null;

	}

}
